part of 'dip_nav_router.dart';

/// A delegate that is used by the [Router] widget to build and configure a
/// navigating widget.
class DipNavDelegate extends RouterDelegate<DipNavRoute>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<DipNavRoute> {
  /// The list of [ResultablePage]
  final List<_DipNavResultablePage> _stackPages = [];

  /// The list of [DipNavRoute]
  final List<DipNavRoute> routes;

  /// The [DipNavRoute]
  final DipNavRoute initRoute;

  /// The [DipNavRoute]
  final DipNavRoute? notFoundRoute;

  final List<NavigatorObserver>? dipNavObservers;
  final TransitionDelegate<dynamic>? transitionDelegate;

  DipNavDelegate(
      {required this.initRoute,
      this.notFoundRoute,
      this.routes = const [],
      this.dipNavObservers,
      this.transitionDelegate}) {
    //_addRoute(initRoute);
  }

  @override
  Future<void> setInitialRoutePath(DipNavRoute configuration) {
    if (_stackPages.isEmpty) return setNewRoutePath(configuration);
    return Future.value();
  }

  /// Called by the [Router] at startup with the structure that the
  /// [RouteInformationParser] obtained from parsing the initial route.
  @override
  Widget build(BuildContext context) {
    if (_stackPages.isEmpty) {
      return Container(color: Colors.white);
    }
    return Navigator(
      key: navigatorKey,
      onPopPage: _onPopPage,
      pages: List.from(_stackPages),
      //observers: List.from(dipNavObservers ?? []),
      transitionDelegate:
          transitionDelegate ?? const DefaultTransitionDelegate<dynamic>(),
    );
  }

  @override
  GlobalKey<NavigatorState> get navigatorKey => GlobalKey<NavigatorState>();

  @override
  Future<void> setNewRoutePath(DipNavRoute configuration) {
    _stackPages.clear();
    return pushRoute(configuration);
  }

  /// Called by the [Router] when it detects a route information may have
  /// changed as a result of rebuild.
  @override
  DipNavRoute? get currentConfiguration {
    return _stackPages.isNotEmpty ? _stackPages.last.route : null;
  }

  Future<T?> pushRoute<T>(
    DipNavRoute route, {
    bool removeUntil = false,
    bool replaceCurrent = false,
    bool rebuild = true,
  }) async {
    assert(!(removeUntil && replaceCurrent),
        'Only removeUntil or replaceCurrent should by true!');
    if (removeUntil) {
      _stackPages.clear();
    } else if (replaceCurrent && _stackPages.isNotEmpty) {
      _stackPages.removeLast();
    }
    _addRoute(route);
    if (rebuild) {
      notifyListeners();
    }
    return Future.value();
  }

  @override
  Future<bool> popRoute() {
    if (_stackPages.length > 1) {
      _removePage(_stackPages.last);
      return Future.value(true);
    }
    return Future.value(false);
  }

  bool canPop() {
    return _stackPages.length > 1;
  }

  bool _onPopPage(Route<dynamic> route, dynamic result) {
    final didPop = route.didPop(result);
    if (!didPop) {
      return false;
    }
    _stackPages.remove(route.settings);
    notifyListeners();
    return true;
  }

  void _removePage(_DipNavResultablePage page) {
    _stackPages.remove(page);
    notifyListeners();
  }

  _DipNavResultablePage _createPage(DipNavRoute route) {
    return _DipNavResultablePage(
      route: route,
    );
  }

  void _addRoute(DipNavRoute route) {
    _stackPages.add(
      _createPage(route),
    );
  }
}
